@@ -1,6 +1,6 @@
 /*
   Mode switching tool for controlling flip flop (multiple device) USB gear
-  Version 1.1.4, 2010/08/21
+  Version 1.1.6, 2010/12/22
 
   Copyright (C) 2007, 2008, 2009, 2010 Josua Dietze (mail to "usb_admin" at the
   domain from the README; please do not post the complete address to the Net!
@@ -42,7 +42,7 @@
 
 /* Recommended tab size: 4 */
 
-#define VERSION "1.1.4"
+#define VERSION "1.1.6"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -76,13 +76,13 @@ char *TempPP=NULL;
 struct usb_device *dev;
 struct usb_dev_handle *devh;
 
-int DefaultVendor=0, DefaultProduct=0, TargetVendor=0, TargetProduct=0, TargetClass=0;
-int MessageEndpoint=0, ResponseEndpoint=0, defaultClass=0, MessageDelay=0;
+int DefaultVendor=0, DefaultProduct=0, TargetVendor=0, TargetProduct=-1, TargetClass=0;
+int MessageEndpoint=0, ResponseEndpoint=0, ReleaseDelay=0;
 int targetDeviceCount=0;
 int devnum=-1, busnum=-1;
 int ret;
 
-char DetachStorageOnly=0, HuaweiMode=0, SierraMode=0, SonyMode=0, GCTMode=0;
+char DetachStorageOnly=0, HuaweiMode=0, SierraMode=0, SonyMode=0, GCTMode=0, KobilMode=0;
 char verbose=0, show_progress=1, ResetUSB=0, CheckSuccess=0, config_read=0;
 char NeedResponse=0, NoDriverLoading=0, InquireDevice=1, sysmode=0;
 
@@ -96,7 +96,7 @@ char ByteString[LINE_DIM/2];
 char buffer[BUF_SIZE];
 
 /* Settable Interface and Configuration (for debugging mostly) (jmw) */
-int Interface = 0, Configuration = -1, AltSetting = -1;
+int Interface = 0, Configuration = 0, AltSetting = -1;
 
 
 static struct option long_options[] = {
@@ -111,12 +111,13 @@ static struct option long_options[] = {
 	{"message-content",		required_argument, 0, 'M'},
 	{"message-content2",	required_argument, 0, '2'},
 	{"message-content3",	required_argument, 0, '3'},
-	{"message-delay",		required_argument, 0, 'w'},
+	{"release-delay",		required_argument, 0, 'w'},
 	{"response-endpoint",	required_argument, 0, 'r'},
 	{"detach-only",			no_argument, 0, 'd'},
 	{"huawei-mode",			no_argument, 0, 'H'},
 	{"sierra-mode",			no_argument, 0, 'S'},
 	{"sony-mode",			no_argument, 0, 'O'},
+	{"kobil-mode",			no_argument, 0, 'T'},
 	{"gct-mode",			no_argument, 0, 'G'},
 	{"need-response",		no_argument, 0, 'n'},
 	{"reset-usb",			no_argument, 0, 'R'},
@@ -135,7 +136,7 @@ static struct option long_options[] = {
 
 void readConfigFile(const char *configFilename)
 {
-	if (verbose) printf("Reading config file: %s\n", configFilename);
+	if (verbose) printf("\nReading config file: %s\n", configFilename);
 	ParseParamHex(configFilename, TargetVendor);
 	ParseParamHex(configFilename, TargetProduct);
 	ParseParamString(configFilename, TargetProductList);
@@ -147,12 +148,13 @@ void readConfigFile(const char *configFilename)
 	ParseParamBool(configFilename, SierraMode);
 	ParseParamBool(configFilename, SonyMode);
 	ParseParamBool(configFilename, GCTMode);
+	ParseParamBool(configFilename, KobilMode);
 	ParseParamBool(configFilename, NoDriverLoading);
 	ParseParamHex(configFilename, MessageEndpoint);
 	ParseParamString(configFilename, MessageContent);
 	ParseParamString(configFilename, MessageContent2);
 	ParseParamString(configFilename, MessageContent3);
-	ParseParamInt(configFilename, MessageDelay);
+	ParseParamInt(configFilename, ReleaseDelay);
 	ParseParamHex(configFilename, NeedResponse);
 	ParseParamHex(configFilename, ResponseEndpoint);
 	ParseParamHex(configFilename, ResetUSB);
@@ -163,9 +165,8 @@ void readConfigFile(const char *configFilename)
 	ParseParamHex(configFilename, AltSetting);
 
 	/* TargetProductList has priority over TargetProduct */
-	if (strlen(TargetProductList))
-	{
-		TargetProduct = 0;
+	if (TargetProduct != -1 && TargetProductList[0] != '\0') {
+		TargetProduct = -1;
 		SHOW_PROGRESS("Warning: TargetProductList overrides TargetProduct!\n");
 	}
 
@@ -187,7 +188,7 @@ void printConfig()
 		printf ("TargetVendor=   0x%04x\n",		TargetVendor);
 	else
 		printf ("TargetVendor=   not set\n");
-	if ( TargetProduct )
+	if ( TargetProduct > -1 )
 		printf ("TargetProduct=  0x%04x\n",		TargetProduct);
 	else
 		printf ("TargetProduct=  not set\n");
@@ -201,6 +202,7 @@ void printConfig()
 	printf ("SierraMode=%i\n",			(int)SierraMode);
 	printf ("SonyMode=%i\n",			(int)SonyMode);
 	printf ("GCTMode=%i\n",			(int)GCTMode);
+	printf ("KobilMode=%i\n",		(int)KobilMode);
 	if ( MessageEndpoint )
 		printf ("MessageEndpoint=0x%02x\n",	MessageEndpoint);
 	else
@@ -216,7 +218,7 @@ void printConfig()
 	else
 		printf ("ResponseEndpoint= not set\n");
 	printf ("Interface=0x%02x\n",			Interface);
-	if ( Configuration > -1 )
+	if ( Configuration > 0 )
 		printf ("Configuration=0x%02x\n",	Configuration);
 	if ( AltSetting > -1 )
 		printf ("AltSetting=0x%02x\n",	AltSetting);
@@ -248,7 +250,7 @@ int readArguments(int argc, char **argv)
 
 	while (1)
 	{
-		c = getopt_long (argc, argv, "heWQDndHSOGRIv:p:V:P:C:m:M:2:3:w:r:c:i:u:a:s:",
+		c = getopt_long (argc, argv, "heWQDndHSOGTRIv:p:V:P:C:m:M:2:3:w:r:c:i:u:a:s:",
 						long_options, &option_index);
 
 		/* Detect the end of the options. */
@@ -267,7 +269,7 @@ int readArguments(int argc, char **argv)
 			case 'M': strcpy(MessageContent, optarg); break;
 			case '2': strcpy(MessageContent2, optarg); break;
 			case '3': strcpy(MessageContent3, optarg); break;
-			case 'w': MessageDelay = strtol(optarg, NULL, 10); count--; break;
+			case 'w': ReleaseDelay = strtol(optarg, NULL, 10); count--; break;
 			case 'n': NeedResponse = 1; break;
 			case 'r': ResponseEndpoint = strtol(optarg, NULL, 16); break;
 			case 'd': DetachStorageOnly = 1; break;
@@ -275,6 +277,7 @@ int readArguments(int argc, char **argv)
 			case 'S': SierraMode = 1; break;
 			case 'O': SonyMode = 1; break;
 			case 'G': GCTMode = 1; break;
+			case 'T': KobilMode = 1; break;
 			case 'c': readConfigFile(optarg); break;
 			case 'W': verbose = 1; show_progress = 1; count--; break;
 			case 'Q': show_progress = 0; verbose = 0; count--; break;
@@ -309,7 +312,8 @@ int readArguments(int argc, char **argv)
 
 int main(int argc, char **argv)
 {
-	int numDefaults = 0, specialMode = 0, sonySuccess = 0;
+	int numDefaults=0, specialMode=0, sonySuccess=0;
+	int currentConfig=0, defaultClass=0, interfaceClass=0;
 
 	/* Make sure we have empty strings even if not set by config */
 	TargetProductList[0] = '\0';
@@ -365,13 +369,13 @@ int main(int argc, char **argv)
 	SHOW_PROGRESS("\n");
 
 	if (show_progress)
-		if (CheckSuccess && !(TargetVendor || TargetProduct || strlen(TargetProductList)) && !TargetClass)
+		if (CheckSuccess && !(TargetVendor || TargetProduct > -1 || TargetProductList[0] != '\0') && !TargetClass)
 			printf("Note: target parameter missing; success check limited\n");
 
 	/* Count existing target devices, remember for success check */
 	if (TargetVendor || TargetClass) {
 		SHOW_PROGRESS("Looking for target devices ...\n");
-		search_devices(&targetDeviceCount, TargetVendor, TargetProduct, TargetProductList, TargetClass, SEARCH_TARGET);
+		search_devices(&targetDeviceCount, TargetVendor, TargetProduct, TargetProductList, TargetClass, 0, SEARCH_TARGET);
 		if (targetDeviceCount) {
 			SHOW_PROGRESS(" Found devices in target mode or class (%d)\n", targetDeviceCount);
 		} else
@@ -380,11 +384,11 @@ int main(int argc, char **argv)
 
 	/* Count default devices, get the last one found */
 	SHOW_PROGRESS("Looking for default devices ...\n");
-	dev = search_devices(&numDefaults, DefaultVendor, DefaultProduct, "\0", TargetClass, SEARCH_DEFAULT);
+	dev = search_devices(&numDefaults, DefaultVendor, DefaultProduct, "\0", TargetClass, Configuration, SEARCH_DEFAULT);
 	if (numDefaults) {
-		SHOW_PROGRESS(" Found devices in default mode or class (%d)\n", numDefaults);
+		SHOW_PROGRESS(" Found devices in default mode, class or configuration (%d)\n", numDefaults);
 	} else {
-		SHOW_PROGRESS(" No devices in default mode or class found. Nothing to do. Bye.\n\n");
+		SHOW_PROGRESS(" No devices in default mode found. Nothing to do. Bye.\n\n");
 		exit(0);
 	}
 	if (dev != NULL) {
@@ -397,12 +401,21 @@ int main(int argc, char **argv)
 		exit(0);
 	}
 
+	/* Get current configuration of default device */
+	currentConfig = get_current_configuration(devh);
+
 	/* Get class of default device/interface */
 	defaultClass = dev->descriptor.bDeviceClass;
+	interfaceClass = get_interface0_class(dev, currentConfig);
+	if (interfaceClass == -1) {
+		fprintf(stderr, "Error: getting the interface class failed. Aborting.\n\n");
+		exit(1);
+	}
+
 	if (defaultClass == 0)
-		defaultClass = dev->config[0].interface[0].altsetting[0].bInterfaceClass;
-	else 
-		if (dev->config[0].interface[0].altsetting[0].bInterfaceClass == 8 && defaultClass != 8) {
+		defaultClass = interfaceClass;
+	else
+		if (interfaceClass == 8 && defaultClass != 8) {
 			/* Weird device with default class other than 0 and differing interface class */
 			SHOW_PROGRESS("Ambiguous Class/InterfaceClass: 0x%02x/0x08\n", defaultClass);
 			defaultClass = 8;
@@ -454,13 +467,13 @@ int main(int argc, char **argv)
 	}
 
 	/* Some scenarios are exclusive, so check for unwanted combinations */
-	specialMode = DetachStorageOnly + HuaweiMode + SierraMode + SonyMode;
+ 	specialMode = DetachStorageOnly + HuaweiMode + SierraMode + SonyMode + KobilMode;
 	if ( specialMode > 1 ) {
 		SHOW_PROGRESS("Invalid mode combination. Check your configuration. Aborting.\n\n");
 		exit(1);
 	}
 
-	if ( !specialMode && !strlen(MessageContent) && AltSetting == -1 && Configuration == -1 )
+	if ( !specialMode && !strlen(MessageContent) && AltSetting == -1 && Configuration == 0 )
 		SHOW_PROGRESS("Warning: no switching method given.\n");
 
 	/*
@@ -493,6 +506,10 @@ int main(int argc, char **argv)
 		detachDriver();
 		switchGCTMode();
 	}
+	if(KobilMode) {
+		detachDriver();
+		switchKobilMode();
+	}
 	if (SonyMode) {
 		if (CheckSuccess)
 			SHOW_PROGRESS("Note: ignoring CheckSuccess. Separate checks for Sony mode\n");
@@ -509,16 +526,35 @@ int main(int argc, char **argv)
 			SHOW_PROGRESS("Warning: ignoring MessageContent. Can't combine with special mode\n");
 	}
 
-	if (Configuration != -1) {
-		switchConfiguration ();
+	if (Configuration != 0) {
+		if (currentConfig != Configuration) {
+			if (switchConfiguration()) {
+				currentConfig = get_current_configuration(devh);
+				if (currentConfig == Configuration) {
+					SHOW_PROGRESS("The configuration was set successfully\n");
+				} else {
+					SHOW_PROGRESS("Setting the configuration failed\n");
+				}
+			}
+		} else {
+			SHOW_PROGRESS("Target configuration %d already active. Doing nothing\n", currentConfig);
+		}
 	}
 
 	if (AltSetting != -1) {
 		switchAltSetting();
 	}
 
+	/* No "removal" check if these are set */
+	if ((Configuration != 0 || AltSetting != -1) && !ResetUSB) {
+		usb_close(devh);
+		devh = 0;
+	}
+
 	if (ResetUSB) {
 		resetUSB();
+		usb_close(devh);
+		devh = 0;
 	}
 
 	if (CheckSuccess) {
@@ -527,7 +563,10 @@ int main(int argc, char **argv)
 				if (NoDriverLoading)
 					printf("ok:\n");
 				else
-					printf("ok:%04x:%04x\n", TargetVendor, TargetProduct);
+					if (TargetProduct < 1)
+						printf("ok:no_data\n");
+					else
+						printf("ok:%04x:%04x\n", TargetVendor, TargetProduct);
 			}
 		} else
 			if (sysmode)
@@ -720,49 +759,29 @@ int switchSendMessage ()
 
 	if (NeedResponse) {
 		SHOW_PROGRESS("Reading the response to the message (CSW) ...\n");
-		ret = read_bulk(ResponseEndpoint, ByteString, 0x200);
+		ret = read_bulk(ResponseEndpoint, ByteString, 13);
 		if (ret < 0)
 			goto skip;
 	}
 
 	if (strlen(MessageContent2)) {
-		if (MessageDelay) {
-			usb_release_interface(devh, Interface);
-			SHOW_PROGRESS("Delaying next message transfer for %d ms\n", MessageDelay);
-			usleep(MessageDelay*1000);
-			ret = usb_claim_interface(devh, Interface);
-			if (ret != 0) {
-				SHOW_PROGRESS(" Could not reclaim interface\n");
-				goto skip;
-			}
-		}
 		if ( sendMessage(MessageContent2, 2) )
 			goto skip;
 
 		if (NeedResponse) {
 			SHOW_PROGRESS("Reading the response to message 2 ...\n");
-			ret = read_bulk(ResponseEndpoint, ByteString, 0x200);
+			ret = read_bulk(ResponseEndpoint, ByteString, 13);
 			if (ret < 0)
 				goto skip;
 		}
 	}
 
 	if (strlen(MessageContent3)) {
-		if (MessageDelay) {
-			usb_release_interface(devh, Interface);
-			SHOW_PROGRESS("Delaying next message transfer for %d ms\n", MessageDelay);
-			usleep(MessageDelay*1000);
-			ret = usb_claim_interface(devh, Interface);
-			if (ret != 0) {
-				SHOW_PROGRESS(" Could not reclaim interface\n");
-				goto skip;
-			}
-		}
 		if ( sendMessage(MessageContent3, 3) )
 			goto skip;
 		if (NeedResponse) {
 			SHOW_PROGRESS("Reading the response to message 3 ...\n");
-			ret = read_bulk(ResponseEndpoint, ByteString, 0x200);
+			ret = read_bulk(ResponseEndpoint, ByteString, 13);
 			if (ret < 0)
 				goto skip;
 		}
@@ -776,6 +795,11 @@ int switchSendMessage ()
 	ret = usb_clear_halt(devh, MessageEndpoint);
 	if (ret)
 		SHOW_PROGRESS(" Error resetting endpoint: %d\n", ret);
+	usleep(200000);
+	if (ReleaseDelay) {
+		SHOW_PROGRESS("Blocking the interface for %d ms before releasing ...\n", ReleaseDelay);
+		usleep(ReleaseDelay*1000);
+	}
 	ret = usb_release_interface(devh, Interface);
 	if (ret)
 		goto skip;
@@ -869,6 +893,20 @@ void switchGCTMode ()
 }
 
 
+int switchKobilMode() {
+	int ret;
+
+	SHOW_PROGRESS("Sending Kobil control message ...\n");
+	ret = usb_control_msg(devh, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, 0x88, 0, 0, buffer, 8, 1000);
+	if (ret != 0) {
+		fprintf(stderr, "Error: sending Kobil control message failed (error %d). Aborting.\n\n", ret);
+		exit(1);
+	} else
+		SHOW_PROGRESS(" OK, Kobil control message sent\n");
+	return 1;
+}
+
+
 int switchSonyMode ()
 {
 	int i, found, ret;
@@ -899,7 +937,7 @@ int switchSonyMode ()
 		if ( i > 5 ) {
 			usb_find_busses();
 			usb_find_devices();
-			dev = search_devices(&found, DefaultVendor, DefaultProduct, "\0", TargetClass, SEARCH_TARGET);
+			dev = search_devices(&found, DefaultVendor, DefaultProduct, "\0", TargetClass, 0, SEARCH_TARGET);
 		}
 		if ( dev != 0 )
 			break;
@@ -922,7 +960,6 @@ int switchSonyMode ()
 		SHOW_PROGRESS(" device still gone, cancelling\n");
 		return 0;
 	}
-
 	sleep(1);
 
 	SHOW_PROGRESS("Sending Sony control message again ...\n");
@@ -935,7 +972,6 @@ int switchSonyMode ()
 	return 1;
 }
 
-
 /* Detach driver either as the main action or as preparation for other
  * switching methods
  */
@@ -1022,6 +1058,7 @@ int checkSuccess()
 			SHOW_PROGRESS(" Waiting for original device to vanish ...\n");
 
 			ret = usb_claim_interface(devh, Interface);
+			usb_release_interface(devh, Interface);
 			if (ret < 0) {
 				SHOW_PROGRESS(" Original device can't be accessed anymore. Good.\n");
 				if (i == CheckSuccess-1)
@@ -1029,19 +1066,15 @@ int checkSuccess()
 				usb_close(devh);
 				devh = 0;
 				break;
-			} else
-				usb_release_interface(devh, Interface);
-
+			}
 			if (i == CheckSuccess-1) {
 				SHOW_PROGRESS(" Original device still present after the timeout\n\nMode switch most likely failed. Bye.\n\n");
 			} else
 				sleep(1);
 		}
-	else
-		SHOW_PROGRESS(" Original device is gone already, not checking\n");
 
 
-	if ( TargetVendor && (TargetProduct || strlen(TargetProductList)) )
+	if ( TargetVendor && (TargetProduct > -1 || TargetProductList[0] != '\0') ) {
 
 		/* Recount target devices (compare with previous count) if target data is given.
 		 * Target device on the same bus with higher device number is returned,
@@ -1049,9 +1082,17 @@ int checkSuccess()
 		 */
 		for (i=i; i < CheckSuccess; i++) {
 			SHOW_PROGRESS(" Searching for target devices ...\n");
-			usb_find_devices();
-			dev = search_devices(&newTargetCount, TargetVendor, TargetProduct, TargetProductList, TargetClass, SEARCH_TARGET);
+			ret = usb_find_busses();
+			if (ret >= 0)
+				ret = usb_find_devices();
+			if (ret < 0) {
+				SHOW_PROGRESS("Error: libusb1 bug, no more searching, try to work around\n");
+				success = 3;
+				break;
+			}
+			dev = search_devices(&newTargetCount, TargetVendor, TargetProduct, TargetProductList, TargetClass, 0, SEARCH_TARGET);
 			if (dev && (newTargetCount > targetDeviceCount)) {
+				printf("\nFound target device, now opening\n");
 				devh = usb_open(dev);
 				deviceDescription();
 				usb_close(devh);
@@ -1075,7 +1116,7 @@ int checkSuccess()
 			} else
 				sleep(1);
 		}
-	else
+	} else
 		/* No target data given, rely on the vanished device */
 		if (!devh) {
 			SHOW_PROGRESS(" (For a better success check provide target IDs or class)\n");
@@ -1084,6 +1125,12 @@ int checkSuccess()
 		}
 
 	switch (success) {
+		case 3: 
+			if (sysmode)
+				syslog(LOG_NOTICE, "switched to new device, but hit libusb1 bug");
+			TargetProduct = -1;
+			success = 1;
+			break;
 		case 2: 
 			if (sysmode)
 				syslog(LOG_NOTICE, "switched to %04x:%04x (%s: %s)", TargetVendor, TargetProduct, imanufact, iproduct);
@@ -1106,7 +1153,7 @@ int checkSuccess()
 int write_bulk(int endpoint, char *message, int length)
 {
 	int ret;
-	ret = usb_bulk_write(devh, endpoint, message, length, 100);
+	ret = usb_bulk_write(devh, endpoint, message, length, 3000);
 	if (ret >= 0 ) {
 		SHOW_PROGRESS(" OK, message successfully sent\n");
 	} else
@@ -1121,7 +1168,7 @@ int write_bulk(int endpoint, char *message, int length)
 int read_bulk(int endpoint, char *buffer, int length)
 {
 	int ret;
-	ret = usb_bulk_read(devh, endpoint, buffer, length, 100);
+	ret = usb_bulk_read(devh, endpoint, buffer, length, 3000);
 	usb_bulk_read(devh, endpoint, buffer, 13, 100);
 	if (ret >= 0 ) {
 		SHOW_PROGRESS(" OK, response successfully read (%d bytes).\n", ret);
@@ -1150,12 +1197,13 @@ void release_usb_device(int dummy) {
 /* Iterates over busses and devices, counts the ones with the given
  * ID/class and returns the last one of them
 */
-struct usb_device* search_devices( int *numFound, int vendor, int product, char* productList, int targetClass, int mode)
+struct usb_device* search_devices( int *numFound, int vendor, int product, char* productList, int targetClass, int configuration, int mode)
 {
 	struct usb_bus *bus;
 	char *listcopy, *token, buffer[2];
 	int devClass;
 	struct usb_device* right_dev = NULL;
+	struct usb_dev_handle *testdevh;
 
 	/* only target class given, target vendor and product assumed unchanged */
 	if ( targetClass && !(vendor || product) ) {
@@ -1167,7 +1215,7 @@ struct usb_device* search_devices( int *numFound, int vendor, int product, char*
 	/* Sanity check */
 	if (!vendor || (!product && productList == '\0') )
 		return NULL;
-		
+
 	if (productList != '\0')
 		listcopy = malloc(strlen(productList)+1);
 
@@ -1219,39 +1267,55 @@ struct usb_device* search_devices( int *numFound, int vendor, int product, char*
 				if (product == dev->descriptor.idProduct) {
 					if (verbose)
 						printf ("   found matching product ID\n");
-					if (targetClass == 0) {
+					if (targetClass == 0 && configuration == 0) {
 						(*numFound)++;
 						right_dev = dev;
 						if (verbose)
 							printf ("   adding device\n");
 					} else {
-						devClass = dev->descriptor.bDeviceClass;
-						if (devClass == 0)
-							devClass = dev->config[0].interface[0].altsetting[0].bInterfaceClass;
-						else
-							/* Check for some quirky devices */
-							if (devClass != dev->config[0].interface[0].altsetting[0].bInterfaceClass)
+						if (targetClass != 0) {
+							devClass = dev->descriptor.bDeviceClass;
+							if (devClass == 0)
 								devClass = dev->config[0].interface[0].altsetting[0].bInterfaceClass;
-						if (devClass == targetClass) {
-							if (verbose)
-								printf ("   target class %02x matching\n", targetClass);
-							if (mode == SEARCH_TARGET) {
+							else
+								/* Check for some quirky devices */
+								if (devClass != dev->config[0].interface[0].altsetting[0].bInterfaceClass)
+									devClass = dev->config[0].interface[0].altsetting[0].bInterfaceClass;
+							if (devClass == targetClass) {
+								if (verbose)
+									printf ("   target class %02x matching\n", targetClass);
+								if (mode == SEARCH_TARGET) {
+									(*numFound)++;
+									right_dev = dev;
+									if (verbose)
+										printf ("   adding device\n");
+								} else
+									if (verbose)
+										printf ("   not adding device\n");
+							} else {
+								if (verbose)
+									printf ("   target class %02x not matching\n", targetClass);
+								if (mode == SEARCH_DEFAULT) {
+									(*numFound)++;
+									right_dev = dev;
+									if (verbose)
+										printf ("   adding device\n");
+								}
+							}
+						} else {
+							// check configuration (only if no target class given)
+							testdevh = usb_open(dev);
+							int testconfig = get_current_configuration(testdevh);
+							if (testconfig != configuration) {
+								if (verbose)
+									printf ("   device configuration %d not matching parameter\n", testconfig);
 								(*numFound)++;
 								right_dev = dev;
 								if (verbose)
 									printf ("   adding device\n");
 							} else
 								if (verbose)
-									printf ("   not adding device as default\n");
-						} else {
-							if (verbose)
-								printf ("   target class %02x not matching\n", targetClass);
-							if (mode == SEARCH_DEFAULT) {
-								(*numFound)++;
-								right_dev = dev;
-								if (verbose)
-									printf ("   adding device as default\n");
-							}
+									printf ("   not adding device, target configuration already set\n");
 						}
 					}
 					/* hack: if busnum has other than init value, we are called from
@@ -1312,6 +1376,30 @@ int find_first_bulk_input_endpoint(struct usb_device *dev)
 	return 0;
 }
 
+int get_current_configuration(struct usb_dev_handle* devh)
+{
+	int ret;
+
+	SHOW_PROGRESS("Getting the current device configuration ...\n");
+	ret = usb_control_msg(devh, USB_DIR_IN + USB_TYPE_STANDARD + USB_RECIP_DEVICE, USB_REQ_GET_CONFIGURATION, 0, 0, buffer, 1, 1000);
+	if (ret < 0) {
+		fprintf(stderr, "Error: getting the current configuration failed (error %d). Aborting.\n\n", ret);
+		exit(1);
+	} else {
+		SHOW_PROGRESS(" OK, got current device configuration (%d)\n", buffer[0]);
+		return buffer[0];
+	}
+}
+
+
+int get_interface0_class(struct usb_device *dev, int devconfig)
+{
+	int i;
+	for (i=0; i<dev->descriptor.bNumConfigurations; i++)
+		if (dev->config[i].bConfigurationValue == devconfig)
+			return dev->config[i].interface[0].altsetting[0].bInterfaceClass;
+	return -1;
+}
 
 
 /* Parameter parsing */
@@ -1427,7 +1515,7 @@ int hexstr2bin(const char *hex, char *buffer, int len)
 void printVersion()
 {
 	char* version = VERSION;
-	printf(" * usb_modeswitch: handle USB devices with multiple modes\n");
+	printf("\n * usb_modeswitch: handle USB devices with multiple modes\n");
 	printf(" * Version %s (C) Josua Dietze 2010\n", version);
 	printf(" * Based on libusb0 (0.1.12 and above)\n\n");
 	printf(" ! PLEASE REPORT NEW CONFIGURATIONS !\n\n");
@@ -1453,6 +1541,7 @@ void printHelp()
 	printf (" -S, --sierra-mode             apply a special procedure\n");
 	printf (" -O, --sony-mode               apply a special procedure\n");
 	printf (" -G, --gct-mode                apply a special procedure\n");
+	printf (" -T, --kobil-mode              apply a special procedure\n");
 	printf (" -R, --reset-usb               reset the device after all other actions\n");
 	printf (" -Q, --quiet                   don't show progress or error messages\n");
 	printf (" -W, --verbose                 print all settings and debug output\n");
