@@ -1,41 +1,25 @@
-#!/bin/sh
-# next lines for bash, ignored by tclsh, restarting in background \
-( \
-count=120; \
-while [ $count != 0 ]; do \
-	if [ ! -e "/usr/bin/tclsh" ]; then \
-		sleep 1; \
-		count=$(($count - 1)); \
-	else \
-		exec /usr/bin/tclsh "$0" "$@" 2>/dev/null & \
-		exit 0; \
-	fi; \
-done; \
-) & \
-exit 0
-
+#!/usr/bin/tclsh
 
 # Wrapper (tcl) for usb_modeswitch, called from
 # /lib/udev/rules.d/40-usb_modeswitch.rules
 # (part of data pack "usb-modeswitch-data") via
 # /lib/udev/usb_modeswitch
 #
-# Does ID check on hotplugged USB devices and calls the
-# mode switching program with the matching parameter file
-# from /etc/usb_modeswitch.d
+# Does ID check on newly discovered USB devices and calls
+# the mode switching program with the matching parameter
+# file from /etc/usb_modeswitch.d
 #
-# Part of usb-modeswitch-1.1.4 package
+# Part of usb-modeswitch-1.1.6 package
 # (C) Josua Dietze 2009, 2010
 
 
 # Setting of the these switches is done in the global config
-# file (/etc/usb_modeswitch.conf)
+# file (/etc/usb_modeswitch.conf) if available
 
 set logging 0
 set noswitching 0
 
 
-
 set env(PATH) "/bin:/usr/bin"
 
 # Execution starts at file bottom
@@ -44,27 +28,20 @@ proc {Main} {argc argv} {
 
 global scsi usb config match wc device logging noswitching settings
 
+set loginit [ParseGlobalConfig]
+
 # The facility to add a symbolic link pointing to the
 # ttyUSB port which provides interrupt transfer, i.e.
 # the port to connect through; returns a symlink name
 # for udev and exits
-# This is run once for every device interface by an
-# udev rule
+# This is run once for every known device interface by
+# an udev rule
 
 if {[lindex $argv 0] == "--symlink-name"} {
-#	set device [clock clicks]
-	puts [SymLinkName [lindex $argv 1]]
-	SafeExit
-}
-
-# The facility to bind the driver on-the-fly after a warm
-# boot; the device is still in modem mode but if the
-# driver was bound by the switching script before (ID
-# not yet added to the driver), the device needs to be
-# bound again
-
-if {[lindex $argv 0] == "--driver-bind"} {
-	CheckDriverBind [lindex $argv 1] [lindex $argv 2] [lindex $argv 3] [lindex $argv 4]
+	if $logging {
+		set device [clock clicks]
+	}
+	puts -nonewline [SymLinkName [lindex $argv 1]]
 	SafeExit
 }
 
@@ -85,9 +62,7 @@ if [string length [lindex $argList 1]] {
 	set device "noname"
 }
 
-ParseGlobalConfig
-
-Log "raw args from udev: $argv"
+Log "raw args from udev: $argv\n\n$loginit"
 
 if {$device == "noname"} {
 	Log "No data from udev. Exiting"
@@ -282,7 +257,6 @@ if {$scsiNeeded && $scsi(vendor)==""} {
 # end and provide a fallback
 
 set report {}
-#set configList [glob -nocomplain $settings(dbdir)/$usb(idVendor):$usb(idProduct)*]
 foreach configuration [lsort -decreasing $configList] {
 
 	# skipping installer leftovers
@@ -296,7 +270,8 @@ foreach configuration [lsort -decreasing $configList] {
 			Log "! matched, now switching"
 		} else {
 			Log "! matched, waiting time set to $config(waitBefore) seconds"
-			after [expr $config(waitBefore) * 1000]
+			append config(waitBefore) "000"
+			after $config(waitBefore)
 			Log " waiting is over, switching starts now"
 		}
 
@@ -307,7 +282,7 @@ foreach configuration [lsort -decreasing $configList] {
 		} else {
 			set report [exec $bindir/usb_modeswitch -I -Q -D -c $settings(tmpConfig) 2>/dev/null]
 		}
-		Log "\nverbose output of usb_modeswitch:"
+		Log "\nVerbose debug output of usb_modeswitch and libusb follows\n(Note that some USB errors are expected in the process)"
 		Log "--------------------------------"
 		Log $report
 		Log "--------------------------------"
@@ -328,7 +303,30 @@ foreach configuration [lsort -decreasing $configList] {
 # simply recounting serial devices under /dev
 
 # If target ID given, driver shall be loaded
-if [regexp -nocase {ok:[0-9a-f]{4}:[0-9a-f]{4}} $report] {
+if [regexp -nocase {ok:[0-9a-f]{4}:[0-9a-f]{4}|ok:no_data} $report] {
+
+	if {![file isdirectory $devdir]} {
+		Log "Device directory in sysfs is gone! Something went wrong, aborting"
+		SafeExit
+	}
+
+	# Give the device annother second if it's not fully back yet
+	if {![file exists $devdir/idProduct]} {
+		after 1000
+	}
+
+	ReadUSBAttrs $devdir
+	if {[string length "$usb(idVendor)$usb(idProduct)"] < 8} {
+		regexp {ok:(\w{4}):(\w{4})} $report d usb(idVendor) usb(idProduct)
+	}
+	set t "$usb(idVendor)$usb(idProduct)"
+	if {[string length $t] != 8 || [string trim $t 0] == ""} {
+		if {$report == "ok:no_data"} {
+			Log "Libusb1 bug prevented device searching, and device ID not found afterwards."
+		}
+		Log "No vendor/product ID found or given, can't continue. Aborting"
+		SafeExit
+	}
 
 	# For general driver loading; TODO: add respective device names.
 	# Presently only useful for HSO devices (which are recounted now)
@@ -350,60 +348,30 @@ if [regexp -nocase {ok:[0-9a-f]{4}:[0-9a-f]{4}} $report] {
 
 	if {[llength $devList1] >= [llength $devList2]} {
 		Log " no new serial devices found"
+		AddToList link_list $usb(idVendor):$usb(idProduct)
+		if {[InBindList $usb(idVendor):$usb(idProduct)] == 0} {
+			Log "Device not in bind_list"
 
-		if {![file isdirectory $devdir]} {
-			Log "Device directory in sysfs is gone! Something went wrong, aborting"
-			SafeExit
-		}
-
-		# Give the device annother second if it's not fully back yet
-		if {![file exists $devdir/idProduct]} {
-			after 1000
-		}
-
-		ReadUSBAttrs $devdir
-		if {[string length "$usb(idVendor)$usb(idProduct)"] < 8} {
-			regexp {ok:(\w{4}):(\w{4})} $report d usb(idVendor) usb(idProduct)
-		}
-		set t "$usb(idVendor)$usb(idProduct)"
-		if {[string length $t] == 8 && [string trim $t 0] != ""} {
-			set idfile $config(driverIDPath)/new_id
-			if {![file exists $idfile]} {
-				Log "\nTrying to load driver \"$config(driverModule)\""
-				set loader /sbin/modprobe
-				Log " loader is: $loader"
-				if [file exists $loader] {
-					if [catch {set result [exec $loader -v $config(driverModule) 2>/dev/null]} err] {
-						Log " Running \"$loader $config(driverModule)\" gave an error:\n  $err"
-					}
-				} else {
-					Log " /sbin/modprobe not found"
-				}
-			}
-			if [file exists $idfile] {
-				Log "Trying to add ID to driver \"$config(driverModule)\""
-				catch {exec logger -p syslog.notice "usb_modeswitch: adding device ID $usb(idVendor):$usb(idProduct) to driver \"$config(driverModule)\"" 2>/dev/null}
-				catch {exec echo "$usb(idVendor) $usb(idProduct)" >$idfile 2>/dev/null}
-				after 600
-				set devList2 [glob -nocomplain /dev/ttyUSB* /dev/ttyACM* /dev/ttyHS*]
-				if {[llength $devList1] >= [llength $devList2]} {
-					Log " still no new serial devices found"
-				} else {
-					AddLastSeen "$usb(idVendor):$usb(idProduct)"
-					Log " driver successfully bound"
-				}
+			CheckDriverBind $usb(idVendor) $usb(idProduct)
+			after 400
+			set devList2 [glob -nocomplain /dev/ttyUSB* /dev/ttyACM* /dev/ttyHS*]
+			if {[llength $devList1] >= [llength $devList2]} {
+				Log " still no new serial devices found"
 			} else {
-				Log " \"$idfile\" not found, can't add ID"
+				Log " driver successfully bound"
+				AddToList bind_list $usb(idVendor):$usb(idProduct)
 			}
 		}
 	} else {
 		Log " new serial devices found, driver has bound"
+		if {[llength [lsearch -all $devList2 *ttyUSB*]] > [llength [lsearch -all $devList1 *ttyUSB*]]} {
+			AddToList link_list $usb(idVendor):$usb(idProduct)
+		}
 	}
 }
 
-
-
 if [regexp {ok:$} $report] {
+	# "NoDriverLoading" was set
 	Log "Doing no driver checking or binding for this device"
 }
 
@@ -530,7 +498,7 @@ while {![eof $rc]} {
 	}
 
 }
-Log "Using global config file: $configFile"
+return "Using global config file: $configFile"
 
 }
 # end of proc {ParseGlobalConfig}
@@ -546,9 +514,15 @@ set rc [open $configFile r]
 set lineList [split [read $rc] \n]
 close $rc
 foreach line $lineList {
-	regexp {DriverModule[[:blank:]]*=[[:blank:]]*"?(\w+)"?} $line d config(driverModule)
-	regexp {DriverIDPath[[:blank:]]*=[[:blank:]]*?"?([/\-\w]+)"?} $line d config(driverIDPath)
-	regexp {WaitBefore[[:blank:]]*=[[:blank:]]*?(\d+)} $line d config(waitBefore)
+	if [regexp {^DriverModule[[:blank:]]*=[[:blank:]]*"?(\w+)"?} [string trim $line] d config(driverModule)] {
+		Log "config: DriverModule set to $config(driverModule)"
+	}
+	if [regexp {^DriverIDPath[[:blank:]]*=[[:blank:]]*?"?([/\-\w]+)"?} [string trim $line] d config(driverIDPath)] {
+		Log "config: DriverIDPath set to $config(driverIDPath)"
+	}
+	if [regexp {^WaitBefore[[:blank:]]*=[[:blank:]]*?([0-9]+)} [string trim $line] d config(waitBefore)] {
+		Log "config: WaitBefore set to $config(waitBefore)"
+	}
 }
 set config(waitBefore) [string trimleft $config(waitBefore) 0]
 
@@ -587,6 +561,7 @@ switch $command {
 		} else {
 			set settings(tmpConfig) $config
 		}
+Log "ConfigGet returns $settings(tmpConfig)"
 		return $settings(tmpConfig)
 	}
 }
@@ -599,9 +574,8 @@ proc {Log} {msg} {
 global wc logging device
 if {$logging == 0} {return}
 if {![info exists wc]} {
-	if [catch {set wc [open /var/log/usb_modeswitch_$device a]} err] {
+	if [catch {set wc [open /var/log/usb_modeswitch_$device w]} err] {
 		set wc "error"
-		puts "Error: Can't write to log file, $err"
 		return
 	}
 	puts $wc "\n\nUSB_ModeSwitch log from [clock format [clock seconds]]\n"
@@ -629,18 +603,26 @@ exit
 # Checking for interrupt endpoint in ttyUSB port (lowest if there is
 # more than one); if found, check for unused "gsmmodem[n]" name.
 # Link for first modem will be "gsmmodem", then "gsmmodem2" and up
-
+#
 proc {SymLinkName} {path} {
+global device
+
+set loginit "* called with --symlink-name: params $path *\n"
 
 # Internal proc, used only here
 proc {hasInterrupt} {ifDir} {
-	if {[llength [glob -nocomplain $ifDir/ttyUSB*]] == 0} {return 0}
+	if {[llength [glob -nocomplain $ifDir/ttyUSB*]] == 0} {
+		Log "  no ttyUSB interface - skip checking endpoints"
+		return 0
+	}
 	foreach epDir [glob -nocomplain $ifDir/ep_*] {
+		Log "  in epDir $epDir ..."
 		if [file exists $epDir/type] {
 			set rc [open $epDir/type r]
 			set type [read $rc]
 			close $rc
 			if [regexp {Interrupt} $type] {
+				Log " $epDir has interrupt transfer type"
 				return 1
 			}
 		}
@@ -653,56 +635,83 @@ proc {hasInterrupt} {ifDir} {
 set linkpath /sys$path/device
 if [file exists $linkpath] {
 	if {[file type $linkpath] == "link"} {
-		set rawpath [file link $linkpath]
+		set rawpath [file readlink $linkpath]
 		set trimpath [regsub -all {\.\./} $rawpath {}]
-		if [file isdirectory /sys/$trimpath] {
+ 		if [file isdirectory /sys/$trimpath] {
 			set path /$trimpath
 		}
 	}
 }
 
-if {![regexp {ttyUSB\d+?} $path myPort]} {
+
+if {![regexp {ttyUSB[0-9]+} $path myPort]} {
+	Log "$loginit\nCould not find port name in path\n $path. Aborting"
 	return ""
 }
-if {![regexp "\\d+\\.(\\d+)/$myPort" $path d myIf]} {
+set device $myPort
+Log "$loginit\nMy name is $myPort"
+
+if {![regexp {usb[0-9]+/([0-9]+-[0-9]+)/} $path d dev_top]} {
+	Log "Could not find device directory in path\n $path. Aborting"
 	return ""
 }
-if {![regexp {usb\d*/(\d+-\d+)/} $path d dev_top]} {
+
+if {![regexp "\[0-9\]+\\.(\[0-9\]+)/$myPort" $path d myIf]} {
+	Log "Could not find interface number in path\n $path. Aborting"
+	return ""
+}
+
+if {![regexp "$dev_top:\[0-9\]" /sys$path ifRoot]} {
+	Log "Could not find interface number in path\n $path. Aborting"
 	return ""
 }
 
 set dirList [split $path /]
 set idx [lsearch $dirList $dev_top]
+
 set devDir /sys[join [lrange $dirList 0 $idx] /]
 
-if {![regexp "$devDir/$dev_top:\[0-9\]" /sys$path ifRoot]} {
-	return ""
-}
+Log "My port is $myPort, my interface is $myIf
+ devDir:  $devDir\n dev_top: $dev_top\nsysPath: /sys$path"
+
+regexp "$devDir/$dev_top:\[0-9\]" /sys$path ifRoot
+
 set ifDir $ifRoot.$myIf
 
-set rightPort 0
+Log "Checking my endpoints in $ifDir"
 if [hasInterrupt $ifDir] {
+	Log "\n--> I am an interrupt port\n"
 	set rightPort 1
+} else {
+	Log "\n--> I am not an interrupt port\n"
+	set rightPort 0
 }
 
 # Unfortunately, there are devices with more than one interrupt
 # port. The assumption so far is that the lowest of these is
 # right. Check all lower interfaces for annother one (if interface)
 # is bigger than 0). If found, don't return any name.
+
 if { $rightPort && ($myIf > 0) } {
+	Log "Looking for lower ports with interrupt endpoints"
 	for {set i 0} {$i < $myIf} {incr i} {
 		set ifDir $ifRoot.$i
+		Log " in ifDir $ifDir ..."
 		if [hasInterrupt $ifDir] {
+			Log "\n--> found an interrupt interface below me\n"
 			set rightPort 0
 			break
 		}
 	}
 }
+
 if {$rightPort == 0} {
+	Log "Return empty name and exit"
 	return ""
 }
 
-# Use first free "gsmmodem[n]" name
+Log "\n--> No interrupt interface below me\n"
+
 cd /dev
 set idx 2
 set symlinkName "gsmmodem"
@@ -713,56 +722,110 @@ while {$idx < 256} {
 	set symlinkName gsmmodem$idx
 	incr idx
 }
+Log "Return symlink name \"$symlinkName\" and exit"
 return $symlinkName
 
 }
 # end of proc {SymLinkName}
 
 
-# Add serial driver after warm boot
-proc {CheckDriverBind} {path vid pid prod} {
+# Load and bind (serial) driver
+#
+proc {CheckDriverBind} {vid pid} {
+global config
 
-if {$vid == ""} {set vid $prod}
-if [regexp {/} $vid] {
-	set id_list [split $vid /]
-	set vid [format %04s [lindex $id_list 0]]
-	set pid [format %04s [lindex $id_list 1]]
+set loader ""
+if [file exists /sbin/modprobe] {
+	set loader /sbin/modprobe
+} else {
+	Log " /sbin/modprobe not found"
 }
 
-set dirList [glob -nocomplain /sys$path/*]
-if [string match *ttyUSB* $dirList] {return}
-
-if {[WasLastSeen $vid:$pid] == 0} {return}
-
-set config(driverModule) "option"
-set config(driverIDPath) "/sys/bus/usb-serial/drivers/option1"
 set idfile $config(driverIDPath)/new_id
 if {![file exists $idfile]} {
-	set loader /sbin/modprobe
-	if {![file exists $loader]} {return}
-	if [catch {exec $loader $config(driverModule) 2>/dev/null}] {return}
-	set i 0
-	while {$i < 50} {
-		if [file exists $idfile] {
-			break
-		}
-		after 20
-		incr i
+	if {$loader == ""} {
+		Log "Can't do anymore without module loader; get \"modtools\"!"
+		return
+	}
+	Log "\nTrying to load driver \"$config(driverModule)\""
+	if [catch {set result [exec $loader -v $config(driverModule)]} err] {
+		Log " Running \"$loader $config(driverModule)\" gave an error:\n  $err"
+	} else {
+		Log " Driver was loaded successfully:\n$result"
+	}
+} else {
+	Log "Driver was loaded already"
+}
+set i 0
+while {$i < 50} {
+	if [file exists $idfile] {
+		break
+	}
+	after 20
+	incr i
+}
+if {$i < 50} {
+	Log "Trying to add ID to driver \"$config(driverModule)\""
+	catch {exec logger -p syslog.notice "usb_modeswitch: adding device ID $vid:$pid to driver \"$config(driverModule)\"" 2>/dev/null}
+	if [catch {exec echo "$vid $pid" >$idfile} err] {
+		Log "Error adding ID to driver: $err"
+	} else {
+		Log " ID added to driver; check for new devices in /dev"
+	}
+} else {
+	Log " \"$idfile\" not found, can't add ID to driver;\n check if kernel version is at least 2.6.27"
+	Log "Falling back to \"usbserial\""
+	set config(driverModule) usbserial
+	Log "\nTrying to unload driver \"$config(driverModule)\""
+	if [catch {exec $loader -r $config(driverModule)} err] {
+		Log " Running \"$loader $config(driverModule)\" gave an error:\n  $err"
+		Log "Can't unload usbserial. No more fallbacks"
+		return
+	}
+	Log "\nTrying to load driver \"usbserial\" with device IDs"
+	if [catch {set result [exec $loader -v usbserial vendor=0x$vid product=0x$pid]} err] {
+		Log " Running \"$loader $config(driverModule)\" gave an error:\n  $err"
+	} else {
+		Log " Driver was loaded successfully:\n$result"
 	}
-	if {$i == 50} {return}
 }
-catch {exec echo "$vid $pid" >$idfile 2>/dev/null}
 
 }
 # end of proc {CheckDriverBind}
 
 
-# Add USB ID to list of devices needing driver binding
-proc {AddLastSeen} {id} {
+# Check if USB ID is listed as needing driver binding
+proc {InBindList} {id} {
 
-set lastseen /etc/usb_modeswitch.d/last_seen
-if [file exists $lastseen] {
-	set rc [open $lastseen r]
+set listfile /etc/usb_modeswitch.d/bind_list
+if {![file exists $listfile]} {return 0}
+set rc [open $listfile r]
+set buffer [read $rc]
+close $rc
+if [string match *$id* $buffer] {
+Log "Found $id in bind_list"
+	return 1
+} else {
+Log "No $id in bind_list"
+	return 0
+}
+
+}
+# end of proc {InBindList}
+
+# Add USB ID to list of devices needing later treatment
+proc {AddToList} {name id} {
+
+if [file exists /etc/usb_modeswitch.d/last_seen] {
+	if [catch {file rename /etc/usb_modeswitch.d/last_seen /etc/usb_modeswitch.d/bind_list} err] {
+		Log "Error renaming the old bind list file ($err)"
+		return
+	}
+}
+
+set listfile /etc/usb_modeswitch.d/$name
+if [file exists $listfile] {
+	set rc [open $listfile r]
 	set buffer [read $rc]
 	close $rc
 	if [string match *$id* $buffer] {
@@ -772,32 +835,13 @@ if [file exists $lastseen] {
 }
 lappend idList $id
 set buffer [join $idList "\n"]
-if [catch {set wc [open $lastseen w]}] {return}
-puts -nonewline $wc $buffer
+if [catch {set wc [open $listfile w]}] {return}
+puts $wc $buffer
 close $wc
 
 }
-# end of proc {AddLastSeen}
-
-
-# Check if USB ID is listed as needing driver binding
-proc {WasLastSeen} {id} {
-
-set lastseen /etc/usb_modeswitch.d/last_seen
-if {![file exists $lastseen]} {return 0}
-set rc [open $lastseen r]
-set buffer [read $rc]
-close $rc
-if [string match *$id* $buffer] {
-	return 1
-} else {
-	return 0
-}
-
-}
-# end of proc {WasLastSeen}
+# end of proc {AddToBindList}
 
 
 # The actual entry point
 Main $argc $argv
-
